@@ -92,6 +92,7 @@ typedef struct {
 	TpHandleType       handle_type;
 	gint               contacts_width;
 	gboolean           has_input_vscroll;
+	gint               topic_width;
 
 	GtkWidget         *widget;
 	GtkWidget         *hpaned;
@@ -100,6 +101,7 @@ typedef struct {
 	GtkWidget         *scrolled_window_input;
 	GtkWidget         *scrolled_window_contacts;
 	GtkWidget         *hbox_topic;
+	GtkWidget         *expander_topic;
 	GtkWidget         *label_topic;
 	GtkWidget         *contact_list_view;
 	GtkWidget         *info_bar_vbox;
@@ -260,13 +262,13 @@ reconnected_connection_ready_cb (TpConnection *connection,
 	switch (priv->handle_type) {
 		case TP_HANDLE_TYPE_CONTACT:
 			empathy_dispatcher_chat_with_contact_id (
-				connection, priv->id,
+				connection, priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION,
 				chat_connect_channel_reconnected,
 				chat);
 			break;
 		case TP_HANDLE_TYPE_ROOM:
 			empathy_dispatcher_join_muc (connection,
-				priv->id,
+				priv->id, EMPATHY_DISPATCHER_NON_USER_ACTION,
 				chat_connect_channel_reconnected,
 				chat);
 			break;
@@ -741,6 +743,7 @@ chat_command_join (EmpathyChat *chat,
 
 			connection = empathy_tp_chat_get_connection (priv->tp_chat);
 			empathy_dispatcher_join_muc (connection, rooms[i],
+						     gtk_get_current_event_time (),
 						     chat_command_join_cb,
 						     chat);
 		}
@@ -765,6 +768,7 @@ chat_command_msg_internal (EmpathyChat *chat,
 	data->message = g_strdup (message);
 	connection = empathy_tp_chat_get_connection (priv->tp_chat);
 	empathy_dispatcher_chat_with_contact_id (connection, contact_id,
+						 gtk_get_current_event_time (),
 						 chat_command_msg_cb,
 						 data);
 }
@@ -1197,6 +1201,88 @@ chat_send_error_cb (EmpathyTpChat          *tp_chat,
 	g_free (str);
 }
 
+/* WARNING: EXPLICIT CONTENT, keep away childrens!
+ *
+ * When a GtkLabel is set to wrap, it assume and hardcoded width. To change
+ * that width we have to set a size request on the label... but that's not
+ * possible because we want the window to be able to shrink, so we MUST request
+ * width of 1. Note that the height of a wrapping label depends on its width.
+ *
+ * To work around that, here is what happens:
+ * 1) size-request is first called, an hardcoded small width is requested by
+ *    GtkLabel, which means also a too big height. We do nothing.
+ * 2) size-allocate is called with the full width available, that's the width
+ *    we really want to make wrap the label. We save that width and restart a
+ *    size-request/size-allocate round.
+ * 3) size-request is called a 2nd time, now we can tell the pango layout its
+ *    width (we can't do that in step 2 because GtkLabel::size-request recreate
+ *    the layout each time). When the layout has its width, we can know the
+ *    height of the label and set its requisition. The width request is set to
+ *    1px to make sure the window can shrink, the layout will fill all the
+ *    available width anyway.
+ */
+static void
+chat_topic_label_size_request_cb (GtkLabel *label,
+				   GtkRequisition *requisition,
+				   EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (gtk_label_get_line_wrap (label) && priv->topic_width > 0) {
+		PangoLayout *layout;
+		PangoRectangle rect;
+		gint ypad;
+
+		layout = gtk_label_get_layout (label);
+		pango_layout_set_width (layout, priv->topic_width * PANGO_SCALE);
+		pango_layout_get_extents (layout, NULL, &rect);
+		gtk_misc_get_padding (GTK_MISC (label), NULL, &ypad);
+
+		requisition->width = 1;
+		requisition->height = PANGO_PIXELS (rect.height) + ypad * 2;
+	}
+}
+
+static void
+chat_topic_label_size_allocate_cb (GtkLabel *label,
+				   GtkAllocation *allocation,
+				   EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (!gtk_label_get_line_wrap (label)) {
+		priv->topic_width = -1;
+
+		if (pango_layout_is_ellipsized (gtk_label_get_layout (label)))
+			gtk_widget_show (priv->expander_topic);
+		else
+			gtk_widget_hide (priv->expander_topic);
+
+		return;
+	}
+
+	if (priv->topic_width != allocation->width) {
+		priv->topic_width = allocation->width;
+		gtk_widget_queue_resize (GTK_WIDGET (label));
+	}
+}
+
+static void
+chat_topic_expander_activate_cb (GtkExpander *expander,
+				 GParamSpec *param_spec,
+				 EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (gtk_expander_get_expanded (expander)) {
+		gtk_label_set_ellipsize (GTK_LABEL (priv->label_topic), PANGO_ELLIPSIZE_NONE);
+		gtk_label_set_line_wrap (GTK_LABEL (priv->label_topic), TRUE);
+	} else {
+		gtk_label_set_ellipsize (GTK_LABEL (priv->label_topic), PANGO_ELLIPSIZE_END);
+		gtk_label_set_line_wrap (GTK_LABEL (priv->label_topic), FALSE);
+	}
+}
+
 static void
 chat_property_changed_cb (EmpathyTpChat *tp_chat,
 			  const gchar   *name,
@@ -1213,11 +1299,16 @@ chat_property_changed_cb (EmpathyTpChat *tp_chat,
 		if (EMP_STR_EMPTY (priv->subject)) {
 			gtk_widget_hide (priv->hbox_topic);
 		} else {
+			gchar *markup_topic;
 			gchar *markup_text;
 
-			markup_text = empathy_add_link_markup (priv->subject);
+			markup_topic = empathy_add_link_markup (priv->subject);
+			markup_text = g_strdup_printf ("<span weight=\"bold\">%s</span> %s",
+				_("Topic:"), markup_topic);
+
 			gtk_label_set_markup (GTK_LABEL (priv->label_topic), markup_text);
 			g_free (markup_text);
+			g_free (markup_topic);
 
 			gtk_widget_show (priv->hbox_topic);
 		}
@@ -1244,13 +1335,10 @@ static void
 chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
                                    EmpathyChat    *chat)
 {
-	EmpathyChatPriv *priv;
 	GtkTextIter     start, end;
 	gchar          *str;
 	gboolean        spell_checker = FALSE;
 
-	priv = GET_PRIV (chat);
-
 	if (gtk_text_buffer_get_char_count (buffer) == 0) {
 		chat_composing_stop (chat);
 	} else {
@@ -2093,7 +2181,11 @@ chat_update_contacts_visibility (EmpathyChat *chat,
 						priv->contacts_width);
 		}
 
-		store = empathy_contact_list_store_new (EMPATHY_CONTACT_LIST (priv->tp_chat));
+		store = empathy_contact_list_store_new (
+				EMPATHY_CONTACT_LIST (priv->tp_chat));
+		empathy_contact_list_store_set_show_groups (
+				EMPATHY_CONTACT_LIST_STORE (store), FALSE);
+
 		priv->contact_list_view = GTK_WIDGET (empathy_contact_list_view_new (store,
 			EMPATHY_CONTACT_LIST_FEATURE_CONTACT_TOOLTIP,
 			EMPATHY_CONTACT_FEATURE_CHAT |
@@ -2212,10 +2304,18 @@ chat_create_ui (EmpathyChat *chat)
 					"scrolled_window_chat", &priv->scrolled_window_chat,
 					"scrolled_window_input", &priv->scrolled_window_input,
 					"hbox_topic", &priv->hbox_topic,
+					"expander_topic", &priv->expander_topic,
 					"label_topic", &priv->label_topic,
 					"scrolled_window_contacts", &priv->scrolled_window_contacts,
 					"info_bar_vbox", &priv->info_bar_vbox,
 					NULL);
+
+	empathy_builder_connect (gui, chat,
+		"expander_topic", "notify::expanded", chat_topic_expander_activate_cb,
+		"label_topic", "size-allocate", chat_topic_label_size_allocate_cb,
+		"label_topic", "size-request", chat_topic_label_size_request_cb,
+		NULL);
+
 	g_free (filename);
 
 	/* Add message view. */
@@ -2255,9 +2355,9 @@ chat_create_ui (EmpathyChat *chat)
 			  G_CALLBACK (chat_input_populate_popup_cb),
 			  chat);
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
-	g_signal_connect (buffer, "changed",
+	tp_g_signal_connect_object  (buffer, "changed",
 			  G_CALLBACK (chat_input_text_buffer_changed_cb),
-			  chat);
+			  chat, 0);
 	gtk_text_buffer_create_tag (buffer, "misspelled",
 				    "underline", PANGO_UNDERLINE_ERROR,
 				    NULL);
@@ -2553,9 +2653,9 @@ account_manager_prepared_cb (GObject *source_object,
 
 	for (l = accounts; l != NULL; l = l->next) {
 		TpAccount *account = l->data;
-		empathy_signal_connect_weak (account, "status-changed",
+		tp_g_signal_connect_object (account, "status-changed",
 					     G_CALLBACK (chat_new_connection_cb),
-					     G_OBJECT (chat));
+					     chat, 0);
 	}
 
 	g_list_free (accounts);
@@ -3012,7 +3112,7 @@ empathy_chat_paste (EmpathyChat *chat)
 	priv = GET_PRIV (chat);
 
 	if (priv->tp_chat == NULL ||
-	    !GTK_WIDGET_IS_SENSITIVE (chat->input_text_view))
+	    !gtk_widget_is_sensitive (chat->input_text_view))
 		return;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
