@@ -26,6 +26,9 @@
 
 #include <telepathy-glib/contact.h>
 #include <telepathy-glib/account.h>
+#ifdef ENABLE_TPL
+#include <telepathy-logger/contact.h>
+#endif /* ENABLE_TPL */
 
 G_BEGIN_DECLS
 
@@ -70,6 +73,10 @@ typedef enum {
 
 GType empathy_contact_get_type (void) G_GNUC_CONST;
 EmpathyContact * empathy_contact_new (TpContact *tp_contact);
+#ifdef ENABLE_TPL
+EmpathyContact * empathy_contact_from_tpl_contact (TpAccount *account,
+    TplContact *tpl_contact);
+#endif /* ENABLE_TPL */
 EmpathyContact * empathy_contact_new_for_log (TpAccount *account,
     const gchar *id, const gchar *name, gboolean is_user);
 TpContact * empathy_contact_get_tp_contact (EmpathyContact *contact);
@@ -125,8 +132,6 @@ gboolean empathy_avatar_save_to_file (EmpathyAvatar *avatar,
     const gchar *filename, GError **error);
 
 GHashTable * empathy_contact_get_location (EmpathyContact *contact);
-void empathy_contact_set_location (EmpathyContact *contact,
-    GHashTable *location);
 gboolean empathy_contact_equal (gconstpointer contact1,
     gconstpointer contact2);
 
