@@ -160,7 +160,7 @@ tp_chat_add (EmpathyContactList *list,
 		 * valid.
 		 * props now belongs to EmpathyDispatcher, don't free it */
 		empathy_dispatcher_create_channel (dispatcher, connection,
-				props, NULL, NULL);
+				props, EMPATHY_DISPATCHER_NON_USER_ACTION, NULL, NULL);
 
 		g_object_unref (dispatcher);
 	} else {
@@ -885,6 +885,14 @@ tp_chat_update_remote_contact (EmpathyTpChat *chat)
 		return;
 	}
 
+	/* This is an MSN chat, but it's the new style where 1-1 chats don't
+	 * have the group interface. If it has the conference interface, then
+	 * it is indeed a MUC. */
+	if (tp_proxy_has_interface_by_id (priv->channel,
+					  EMP_IFACE_QUARK_CHANNEL_INTERFACE_CONFERENCE)) {
+		return;
+	}
+
 	/* This is an MSN-like chat where anyone can join the chat at anytime.
 	 * If there is only one non-self contact member, we are in a private
 	 * chat and we set the "remote-contact" property to that contact. If
@@ -1061,6 +1069,12 @@ tp_chat_got_renamed_contacts_cb (EmpathyTpContactFactory *factory,
 		}
 	}
 
+	if (priv->user == old) {
+		/* We change our nick */
+		g_object_unref (priv->user);
+		priv->user = g_object_ref (new);
+	}
+
 	tp_chat_update_remote_contact (EMPATHY_TP_CHAT (chat));
 	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
 }
@@ -1280,7 +1294,7 @@ tp_chat_constructor (GType                  type,
 		list = empathy_dispatcher_find_requestable_channel_classes (
 			dispatcher, connection,
 			tp_channel_get_channel_type (priv->channel),
-			TP_HANDLE_TYPE_ROOM, NULL);
+			TP_UNKNOWN_HANDLE_TYPE, NULL);
 
 		for (ptr = list; ptr; ptr = ptr->next) {
 			GValueArray *array = ptr->data;
@@ -1828,13 +1842,17 @@ empathy_tp_chat_leave (EmpathyTpChat *self)
 	GArray *array;
 
 	if (!tp_proxy_has_interface_by_id (priv->channel,
-		TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP))
+		TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP)) {
 		empathy_tp_chat_close (self);
+		return;
+	}
 
 	self_handle = tp_channel_group_get_self_handle (priv->channel);
-	if (self_handle == 0)
+	if (self_handle == 0) {
 		/* we are not member of the channel */
 		empathy_tp_chat_close (self);
+		return;
+	}
 
 	array = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
 	g_array_insert_val (array, 0, self_handle);
