@@ -182,6 +182,22 @@ debug_message_list_free (gpointer data)
   g_list_free (list);
 }
 
+static gchar *
+get_active_cm_name (EmpathyDebugWindow *self)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (self);
+  GtkTreeIter iter;
+  gchar *name;
+
+  if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter))
+    return NULL;
+
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
+      COL_CM_NAME, &name, -1);
+
+  return name;
+}
+
 static void
 debug_window_cache_new_message (EmpathyDebugWindow *debug_window,
     gdouble timestamp,
@@ -190,15 +206,11 @@ debug_window_cache_new_message (EmpathyDebugWindow *debug_window,
     const gchar *message)
 {
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
-  GtkTreeIter iter;
   GList *messages;
   DebugMessage *dm;
   char *name;
 
-  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter);
-
-  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
-      COL_CM_NAME, &name, -1);
+  name = get_active_cm_name (debug_window);
   messages = g_hash_table_lookup (priv->all_cms, name);
 
   dm = debug_message_new (timestamp, domain, level, message);
@@ -329,7 +341,6 @@ debug_window_get_messages_cb (TpProxy *proxy,
 {
   EmpathyDebugWindow *debug_window = (EmpathyDebugWindow *) user_data;
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
-  GtkTreeIter iter;
   gchar *name;
   GList *old_messages;
   guint i;
@@ -343,10 +354,7 @@ debug_window_get_messages_cb (TpProxy *proxy,
 
   debug_window_set_toolbar_sensitivity (debug_window, TRUE);
 
-  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter);
-
-  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
-      COL_CM_NAME, &name, -1);
+  name = get_active_cm_name (debug_window);
   old_messages = g_hash_table_lookup (priv->all_cms, name);
 
   /* we call get_messages either when a new CM is added or
@@ -746,8 +754,8 @@ debug_window_fill_cm_chooser (EmpathyDebugWindow *debug_window)
   /* Add empathy */
   gtk_list_store_append (priv->cms, &iter);
   gtk_list_store_set (priv->cms, &iter,
-      COL_CM_NAME, "empathy",
-      COL_CM_UNIQUE_NAME, EMPATHY_CLIENT_NAME,
+      COL_CM_NAME, _(PACKAGE_NAME),
+      COL_CM_UNIQUE_NAME, "org.gnome."PACKAGE_NAME,
       -1);
   gtk_combo_box_set_active (GTK_COMBO_BOX (priv->cm_chooser), 0);
 
@@ -768,12 +776,12 @@ debug_window_fill_cm_chooser (EmpathyDebugWindow *debug_window)
 }
 
 static void
-debug_window_pause_toggled_cb (GtkToggleToolButton *pause,
+debug_window_pause_toggled_cb (GtkToggleToolButton *pause_,
     EmpathyDebugWindow *debug_window)
 {
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
 
-  priv->paused = gtk_toggle_tool_button_get_active (pause);
+  priv->paused = gtk_toggle_tool_button_get_active (pause_);
 
   debug_window_set_enabled (debug_window, !priv->paused);
 }
@@ -1062,6 +1070,10 @@ debug_window_save_clicked_cb (GtkToolButton *tool_button,
     EmpathyDebugWindow *debug_window)
 {
   GtkWidget *file_chooser;
+  gchar *name, *tmp = NULL;
+  char time_str[32];
+  time_t t;
+  struct tm *tm_s;
 
   file_chooser = gtk_file_chooser_dialog_new (_("Save"),
       GTK_WINDOW (debug_window), GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -1073,6 +1085,26 @@ debug_window_save_clicked_cb (GtkToolButton *tool_button,
   gtk_file_chooser_set_do_overwrite_confirmation (
       GTK_FILE_CHOOSER (file_chooser), TRUE);
 
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser),
+      g_get_tmp_dir ());
+
+  name = get_active_cm_name (debug_window);
+
+  t = time (NULL);
+  tm_s = localtime (&t);
+  if (tm_s != NULL)
+    {
+      if (strftime(time_str, sizeof (time_str), "%d-%m-%y_%H-%M-%S", tm_s))
+        tmp = g_strdup_printf ("%s-%s.log", name, time_str);
+    }
+
+  if (tmp == NULL)
+    tmp = g_strdup_printf ("%s.log", name);
+  g_free (name);
+
+  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_chooser), tmp);
+  g_free (tmp);
+
   g_signal_connect (file_chooser, "response",
       G_CALLBACK (debug_window_save_file_chooser_response_cb),
       debug_window);
@@ -1165,6 +1197,37 @@ debug_window_key_press_event_cb (GtkWidget *widget,
   return FALSE;
 }
 
+static gboolean
+tree_view_search_equal_func_cb (GtkTreeModel *model,
+    gint column,
+    const gchar *key,
+    GtkTreeIter *iter,
+    gpointer search_data)
+{
+  gchar *str;
+  gint key_len;
+  gint len;
+  gint i;
+  gboolean ret = TRUE; /* The return value is counter-intuitive */
+
+  gtk_tree_model_get (model, iter, column, &str, -1);
+
+  key_len = strlen (key);
+  len = strlen (str) - key_len;
+
+  for (i = 0; i <= len; ++i)
+    {
+      if (!g_ascii_strncasecmp (key, str + i, key_len))
+        {
+          ret = FALSE;
+          break;
+        }
+    }
+
+  g_free (str);
+  return ret;
+}
+
 static GObject *
 debug_window_constructor (GType type,
     guint n_construct_params,
@@ -1373,6 +1436,11 @@ debug_window_constructor (GType type,
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (priv->view), priv->store_filter);
 
+  gtk_tree_view_set_search_column (GTK_TREE_VIEW (priv->view),
+      COL_DEBUG_MESSAGE);
+  gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (priv->view),
+      tree_view_search_equal_func_cb, NULL, NULL);
+
   /* Scrolled window */
   priv->scrolled_win = g_object_ref (gtk_scrolled_window_new (NULL, NULL));
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->scrolled_win),
