@@ -83,8 +83,9 @@ status_icon_notification_closed_cb (NotifyNotification *notification,
 {
 	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
 
-	if (priv->notification) {
-		g_object_unref (priv->notification);
+	g_object_unref (notification);
+
+	if (priv->notification == notification) {
 		priv->notification = NULL;
 	}
 
@@ -99,9 +100,8 @@ status_icon_notification_closed_cb (NotifyNotification *notification,
 static void
 notification_close_helper (EmpathyStatusIconPriv *priv)
 {
-	if (priv->notification) {
+	if (priv->notification != NULL) {
 		notify_notification_close (priv->notification, NULL);
-		g_object_unref (priv->notification);
 		priv->notification = NULL;
 	}
 }
@@ -131,24 +131,49 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 
 	if (priv->event) {
 		gchar *message_esc = NULL;
+		gboolean has_x_canonical_append;
+		NotifyNotification *notification = priv->notification;
 
 		if (priv->event->message != NULL)
 			message_esc = g_markup_escape_text (priv->event->message, -1);
 
-		if (priv->notification) {
-			notify_notification_update (priv->notification,
+		has_x_canonical_append =
+				empathy_notify_manager_has_capability (priv->notify_mgr,
+					EMPATHY_NOTIFY_MANAGER_CAP_X_CANONICAL_APPEND);
+
+		if (notification != NULL && ! has_x_canonical_append) {
+			/* if the notification server supports x-canonical-append, it is
+			   better to not use notify_notification_update to avoid
+			   overwriting the current notification message */
+			notify_notification_update (notification,
 						    priv->event->header, message_esc,
 						    NULL);
 		} else {
-			priv->notification = notify_notification_new_with_status_icon
+			/* if the notification server supports x-canonical-append,
+			   the hint will be added, so that the message from the
+			   just created notification will be automatically appended
+			   to an existing notification with the same title.
+			   In this way the previous message will not be lost: the new
+			   message will appear below it, in the same notification */
+			notification = notify_notification_new_with_status_icon
 				(priv->event->header, message_esc, NULL, priv->icon);
-			notify_notification_set_timeout (priv->notification,
+
+			if (priv->notification == NULL) {
+				priv->notification = notification;
+			}
+
+			notify_notification_set_timeout (notification,
 							 NOTIFY_EXPIRES_DEFAULT);
 
+			if (has_x_canonical_append) {
+				notify_notification_set_hint_string (notification,
+					EMPATHY_NOTIFY_MANAGER_CAP_X_CANONICAL_APPEND, "");
+			}
+
 			if (empathy_notify_manager_has_capability (priv->notify_mgr,
 			           EMPATHY_NOTIFY_MANAGER_CAP_ACTIONS) &&
 			           priv->event->type != EMPATHY_EVENT_TYPE_PRESENCE) {
-				notify_notification_add_action (priv->notification,
+				notify_notification_add_action (notification,
 					"respond",
 					_("Respond"),
 					(NotifyActionCallback) notification_action_cb,
@@ -156,7 +181,7 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 					NULL);
 			}
 
-			g_signal_connect (priv->notification, "closed",
+			g_signal_connect (notification, "closed",
 					  G_CALLBACK (status_icon_notification_closed_cb), icon);
 		}
 
@@ -165,12 +190,11 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 								   priv->event->icon_name);
 
 		if (pixbuf != NULL) {
-			notify_notification_set_icon_from_pixbuf (priv->notification,
-							  pixbuf);
+			notify_notification_set_icon_from_pixbuf (notification, pixbuf);
 			g_object_unref (pixbuf);
 		}
 
-		notify_notification_show (priv->notification, NULL);
+		notify_notification_show (notification, NULL);
 
 		g_free (message_esc);
 	} else {
@@ -379,7 +403,6 @@ status_icon_presence_changed_cb (EmpathyStatusIcon *icon)
 
 		if (priv->notification) {
 			notify_notification_close (priv->notification, NULL);
-			g_object_unref (priv->notification);
 			priv->notification = NULL;
 		}
 	}
@@ -578,9 +601,9 @@ account_manager_prepared_cb (GObject *source_object,
 
 	list = tp_account_manager_get_valid_accounts (account_manager);
 	for (l = list; l != NULL; l = l->next) {
-		empathy_signal_connect_weak (l->data, "status-changed",
+		tp_g_signal_connect_object (l->data, "status-changed",
 					     G_CALLBACK (status_icon_status_changed_cb),
-					     G_OBJECT (icon));
+					     icon, 0);
 	}
 	g_list_free (list);
 
@@ -629,6 +652,7 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 			  G_CALLBACK (status_icon_popup_menu_cb),
 			  icon);
 
+	priv->notification = NULL;
 	priv->notify_mgr = empathy_notify_manager_dup_singleton ();
 }
 
