@@ -72,6 +72,7 @@
 #include "empathy-import-mc4-accounts.h"
 #include "empathy-accounts-common.h"
 #include "empathy-accounts-dialog.h"
+#include "empathy-chat-manager.h"
 #include "empathy-status-icon.h"
 #include "empathy-call-window.h"
 #include "empathy-chat-window.h"
@@ -132,7 +133,8 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
            * (a GtkNotebook) when we'll call empathy_chat_window_present_chat */
         }
 
-      empathy_chat_window_present_chat (chat);
+      empathy_chat_window_present_chat (chat,
+          empathy_dispatch_operation_get_user_action_time (operation));
 
       empathy_dispatch_operation_claim (operation);
     }
@@ -240,24 +242,12 @@ migrate_config_to_xdg_dir (void)
   g_free (old_dir);
 }
 
-static void
-accounts_application_exited_cb (GPid pid,
-    gint status,
-    gpointer data)
-{
-  if (status)
-    {
-      g_warning ("accounts application exited with status %d: '%s'",
-          status, g_strerror (status));
-    }
-}
-
 static void
 show_accounts_ui (GdkScreen *screen,
     gboolean if_needed)
 {
   empathy_accounts_dialog_show_application (screen,
-      accounts_application_exited_cb, NULL, NULL, if_needed, start_hidden);
+      NULL, if_needed, start_hidden);
 }
 
 static UniqueResponse
@@ -492,7 +482,8 @@ account_status_changed_cb (TpAccount *account,
     return;
 
   empathy_dispatcher_join_muc (conn,
-      empathy_chatroom_get_room (room), NULL, NULL);
+      empathy_chatroom_get_room (room), EMPATHY_DISPATCHER_NON_USER_ACTION,
+      NULL, NULL);
 }
 
 static void
@@ -540,7 +531,8 @@ account_manager_chatroom_ready_cb (GObject *source_object,
           else
             {
               empathy_dispatcher_join_muc (conn,
-                  empathy_chatroom_get_room (room), NULL, NULL);
+                  empathy_chatroom_get_room (room),
+                  EMPATHY_DISPATCHER_NON_USER_ACTION, NULL, NULL);
             }
         }
 
@@ -561,6 +553,18 @@ chatroom_manager_ready_cb (EmpathyChatroomManager *chatroom_manager,
       account_manager_chatroom_ready_cb, chatroom_manager);
 }
 
+static void
+empathy_idle_set_auto_away_cb (EmpathyConf *conf,
+				const gchar *key,
+				gpointer user_data)
+{
+	gboolean autoaway;
+	EmpathyIdle *idle = user_data;
+
+	empathy_conf_get_bool (conf, key, &autoaway);
+	empathy_idle_set_auto_away (idle, autoaway);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -581,13 +585,14 @@ main (int argc, char *argv[])
   GtkWidget *window;
   EmpathyIdle *idle;
   EmpathyConnectivity *connectivity;
+  EmpathyChatManager *chat_manager;
   GError *error = NULL;
   UniqueApp *unique_app;
   gboolean chatroom_manager_ready;
-
+  gboolean autoaway = TRUE;
 #ifdef ENABLE_DEBUG
   TpDebugSender *debug_sender;
-#endif /* ENABLE_TPL */
+#endif
 
   GOptionContext *optcontext;
   GOptionEntry options[] = {
@@ -653,7 +658,16 @@ main (int argc, char *argv[])
 
   /* Setting up Idle */
   idle = empathy_idle_dup_singleton ();
-  empathy_idle_set_auto_away (idle, TRUE);
+
+  empathy_conf_get_bool (empathy_conf_get (),
+      EMPATHY_PREFS_AUTOAWAY, &autoaway);
+
+  empathy_conf_notify_add (empathy_conf_get (),
+			   EMPATHY_PREFS_AUTOAWAY,
+			   empathy_idle_set_auto_away_cb,
+			   idle);
+
+  empathy_idle_set_auto_away (idle, autoaway);
 
   /* Setting up Connectivity */
   connectivity = empathy_connectivity_dup_singleton ();
@@ -677,6 +691,9 @@ main (int argc, char *argv[])
   window = empathy_main_window_show ();
   icon = empathy_status_icon_new (GTK_WINDOW (window), start_hidden);
 
+  /* Chat manager */
+  chat_manager = empathy_chat_manager_dup_singleton ();
+
   g_signal_connect (unique_app, "message-received",
       G_CALLBACK (unique_app_message_cb), window);
 
@@ -726,6 +743,7 @@ main (int argc, char *argv[])
   g_object_unref (debug_sender);
 #endif
 
+  g_object_unref (chat_manager);
   g_object_unref (idle);
   g_object_unref (connectivity);
   g_object_unref (icon);
